@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # thoth-adviser
-# Copyright(C) 2020 Fridolin Pokorny
+# Copyright(C) 2020 - 2021 Fridolin Pokorny
 #
 # This program is free software: you can redistribute it and / or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 """A boot to check for solved software environment before running any resolution."""
 
 import logging
-from typing import Optional
+from typing import Generator
 from typing import Dict
 from typing import Any
 from typing import TYPE_CHECKING
@@ -42,14 +42,13 @@ class SolvedSoftwareEnvironmentBoot(Boot):
     _JUSTIFICATION_LINK = jl("solved_sw_env")
 
     @classmethod
-    def should_include(cls, builder_context: "PipelineBuilderContext") -> Optional[Dict[str, Any]]:
+    def should_include(cls, builder_context: "PipelineBuilderContext") -> Generator[Dict[str, Any], None, None]:
         """Register self, always."""
-        if not builder_context.project.runtime_environment.is_fully_specified():
+        if builder_context.project.runtime_environment.is_fully_specified() and not builder_context.is_included(cls):
+            yield {}
             return None
 
-        if not builder_context.is_included(cls):
-            return {}
-
+        yield from ()
         return None
 
     def run(self) -> None:
@@ -65,7 +64,15 @@ class SolvedSoftwareEnvironmentBoot(Boot):
         msg = (
             f"No observations found for {runtime_environment.operating_system.name!r} in "
             f"version {runtime_environment.operating_system.version!r} using "
-            f"Python {runtime_environment.python_version!r} - see {self._JUSTIFICATION_LINK}"
+            f"Python {runtime_environment.python_version!r}"
+        )
+
+        self.context.stack_info.append(
+            {
+                "type": "ERROR",
+                "message": msg,
+                "link": self._JUSTIFICATION_LINK,
+            }
         )
 
         _LOGGER.warning("%s - %s", msg, self._JUSTIFICATION_LINK)
@@ -77,6 +84,14 @@ class SolvedSoftwareEnvironmentBoot(Boot):
             configurations,
             key=lambda i: (i["os_name"], i["os_version"], i["python_version"]),
         ):
+            self.context.stack_info.append(
+                {
+                    "message": f"Consider using {conf['os_name']!r} in version {conf['os_version']!r} "
+                    f"with Python {conf['python_version']}",
+                    "type": "ERROR",
+                    "link": self._JUSTIFICATION_LINK,
+                }
+            )
             _LOGGER.warning("{:<16} {:<16} {:<8}".format(conf["os_name"], conf["os_version"], conf["python_version"]))
 
         raise NotAcceptable(msg)
