@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # thoth-adviser
-# Copyright(C) 2018, 2019, 2020 Fridolin Pokorny
+# Copyright(C) 2018 - 2021 Fridolin Pokorny
 #
 # This program is free software: you can redistribute it and / or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,10 +21,11 @@
 import json
 import logging
 import os
-import time
+import pickle
 import random
+import sys
+import time
 from functools import partial
-from pathlib import Path
 from typing import Any
 from typing import Callable
 from typing import Dict
@@ -38,19 +39,21 @@ import termial_random
 from thoth.analyzer import print_command_result
 from thoth.common import init_logging
 from thoth.common import RuntimeEnvironment
+from thoth.python import Constraints
 from thoth.python import Pipfile
 from thoth.python import PipfileLock
 from thoth.python import Project
-from thoth.python.exceptions import UnsupportedConfiguration
+from thoth.python.exceptions import UnsupportedConfigurationError
 
 from thoth.adviser.dependency_monkey import DependencyMonkey
 from thoth.adviser.digests_fetcher import GraphDigestsFetcher
-from thoth.adviser.pipeline_builder import PipelineBuilder
 from thoth.adviser.enums import DecisionType
 from thoth.adviser.enums import PythonRecommendationOutput
 from thoth.adviser.enums import RecommendationType
 from thoth.adviser.exceptions import AdviserException
 from thoth.adviser.exceptions import InternalError
+from thoth.adviser.pipeline_builder import PipelineBuilder
+from thoth.adviser.prescription import Prescription
 from thoth.adviser import Resolver
 from thoth.adviser import __title__ as analyzer_name
 from thoth.adviser import __version__ as analyzer_version
@@ -84,7 +87,9 @@ def _print_version(ctx: click.Context, _, value: str):
 def _instantiate_project(
     requirements: str,
     requirements_locked: Optional[str] = None,
+    *,
     runtime_environment: RuntimeEnvironment = None,
+    constraints: Optional[str] = None,
 ):
     """Create Project instance based on arguments passed to CLI."""
     try:
@@ -103,11 +108,29 @@ def _instantiate_project(
             requirements_locked = requirements_locked.replace("\\n", "\n")
 
     pipfile = Pipfile.from_string(requirements)
-    pipfile_lock = PipfileLock.from_string(requirements_locked, pipfile) if requirements_locked else None
+    pipfile_lock = None
+    if requirements_locked and requirements_locked != "null":
+        pipfile_lock = PipfileLock.from_string(requirements_locked, pipfile)
+
+    constraints_instance = None
+    if constraints:
+        try:
+            with open(constraints, "r") as constraints_file:
+                constraints_content = constraints_file.read()
+        except (OSError, FileNotFoundError):
+            # We we gather values from env vars, un-escape new lines.
+            constraints_content = constraints.replace("\\n", "\n")
+
+        try:
+            constraints_instance = Constraints.from_dict(json.loads(constraints_content))
+        except json.decoder.JSONDecodeError:
+            constraints_instance = Constraints.from_string(constraints_content)
+
     project = Project(
         pipfile=pipfile,
         pipfile_lock=pipfile_lock,
         runtime_environment=runtime_environment or RuntimeEnvironment.from_dict({}),
+        constraints=constraints_instance or Constraints(),
     )
 
     return project
@@ -267,7 +290,7 @@ def provenance(
             whitelisted_sources=whitelisted_sources,
             digests_fetcher=GraphDigestsFetcher(),
         )
-    except (AdviserException, UnsupportedConfiguration) as exc:
+    except (AdviserException, UnsupportedConfigurationError) as exc:
         if isinstance(exc, InternalError):
             # Re-raise internal exceptions that shouldn't occur here.
             raise
@@ -410,7 +433,36 @@ def provenance(
     default=None,
     type=str,
     metavar="PIPELINE",
-    help="Pipeline configuration supplied in a form of JSON/YAML or a path to a file.",
+    help="Pipeline configuration supplied in a form of JSON/YAML or a path to a file, "
+    "disjoint with pipeline prescription.",
+)
+@click.option(
+    "--prescription",
+    envvar="THOTH_ADVISER_PRESCRIPTION",
+    default=None,
+    type=str,
+    multiple=True,
+    metavar="PRESCRIPTION",
+    help="Pipeline prescription supplied in a form of JSON/YAML or a path to a file, "
+    "disjoint with pipeline configuration. Multiple files can be supplied by using `,' as a delimiter.",
+)
+@click.option(
+    "--constraints",
+    envvar="THOTH_ADVISER_CONSTRAINTS",
+    default=None,
+    type=str,
+    metavar="CONSTRAINTS.txt",
+    help="Constraints to be used during the resolution.",
+)
+@click.option(
+    "--labels",
+    "-l",
+    envvar="THOTH_ADVISER_LABELS",
+    type=str,
+    metavar='{"key1": "val1", "key2": "val2"}',
+    default=None,
+    show_default=True,
+    help="Labels used used to label the request.",
 )
 @click.option(
     "--user-stack-scoring/--no-user-stack-scoring",
@@ -450,32 +502,73 @@ def advise(
     runtime_environment: Optional[str] = None,
     seed: Optional[int] = None,
     pipeline: Optional[str] = None,
+    prescription: Optional[str] = None,
+    constraints: Optional[str] = None,
     user_stack_scoring: bool = True,
     dev: bool = False,
+    labels: Optional[str] = None,
 ):
     """Advise package and package versions in the given stack or on solely package only."""
     parameters = locals()
     parameters.pop("click_ctx")
 
+    if pipeline and prescription:
+        sys.exit("Options --pipeline/--prescription are disjoint")
+
     if library_usage:
         if os.path.isfile(library_usage):
             try:
-                library_usage = json.loads(Path(library_usage).read_text())
+                with open(library_usage, "r") as f:
+                    library_usage = json.load(f)
             except Exception:
                 _LOGGER.error("Failed to load library usage file %r", library_usage)
                 raise
         else:
             library_usage = json.loads(library_usage)
 
+        # Show library usage in the final report.
+        parameters["library_usage"] = library_usage
+
+    labels_dict = {}
+    if labels:
+        if os.path.isfile(labels):
+            try:
+                with open(labels, "r") as f:
+                    labels_dict = json.load(f)
+            except Exception:
+                _LOGGER.error("Failed to load labels file %r", labels)
+                raise
+        else:
+            labels_dict = json.loads(labels)
+
+        # Show labels in the final report.
+        parameters["labels"] = labels_dict
+
     runtime_environment = RuntimeEnvironment.load(runtime_environment)
     recommendation_type = RecommendationType.by_name(recommendation_type)
+    _LOGGER.info("Using recommendation type %s", recommendation_type.name.lower())
+
     requirements_format = PythonRecommendationOutput.by_name(requirements_format)
-    project = _instantiate_project(requirements, requirements_locked, runtime_environment)
-    pipeline_config = None if pipeline is None else PipelineBuilder.load(pipeline)
+    project = _instantiate_project(
+        requirements, requirements_locked, runtime_environment=runtime_environment, constraints=constraints
+    )
+
+    pipeline_config = None
+    if pipeline:
+        pipeline_config = PipelineBuilder.load(pipeline)
 
     parameters["project"] = project.to_dict()
     if pipeline_config is not None:
         parameters["pipeline"] = pipeline_config.to_dict()
+
+    prescription_instance = None
+    if prescription:
+        if len(prescription) == 1:
+            # Click does not support multiple parameters when supplied via env vars. Perform split on delimiter.
+            prescription_instance = Prescription.load(*prescription[0].split(","))
+        else:
+            prescription_instance = Prescription.load(*prescription)
+
     predictor_class, predictor_kwargs = _get_adviser_predictor(predictor, recommendation_type)
     predictor_kwargs = _get_predictor_kwargs(predictor_config) or predictor_kwargs
     predictor_instance = predictor_class(**predictor_kwargs, keep_history=plot is not None)
@@ -494,6 +587,7 @@ def advise(
     resolver = Resolver.get_adviser_instance(
         predictor=predictor_instance,
         project=project,
+        labels=labels_dict,
         library_usage=library_usage,
         recommendation_type=recommendation_type,
         limit=limit,
@@ -501,9 +595,12 @@ def advise(
         beam_width=beam_width,
         limit_latest_versions=limit_latest_versions,
         pipeline_config=pipeline_config,
+        prescription=prescription_instance,
         cli_parameters=parameters,
     )
 
+    del prescription  # No longer needed, garbage collect it.
+
     print_func = _PrintFunc(
         partial(
             print_command_result,
@@ -522,6 +619,7 @@ def advise(
         result_dict={"parameters": parameters},
         with_devel=dev,
         user_stack_scoring=user_stack_scoring,
+        verbose=click_ctx.parent.params.get("verbose", False),
     )
 
     click_ctx.exit(int(exit_code != 0))
@@ -657,7 +755,17 @@ def advise(
     default=None,
     type=str,
     metavar="PIPELINE",
-    help="Pipeline configuration supplied in a form of JSON/YAML or a path to a file.",
+    help="Pipeline configuration supplied in a form of JSON/YAML or a path to a file, "
+    "disjoint with pipeline prescription. Multiple files can be supplied by using `,' as a delimiter.",
+)
+@click.option(
+    "--prescription",
+    envvar="THOTH_ADVISER_PRESCRIPTION",
+    default=None,
+    type=str,
+    metavar="PRESCRIPTION",
+    help="Pipeline prescription supplied in a form of JSON/YAML or a path to a file, "
+    "disjoint with pipeline configuration.",
 )
 @click.option(
     "--dev/--no-dev",
@@ -688,32 +796,51 @@ def dependency_monkey(
     runtime_environment: str = None,
     seed: Optional[int] = None,
     pipeline: Optional[str] = None,
+    prescription: Optional[str] = None,
     dev: bool = False,
 ):
     """Generate software stacks based on all valid resolutions that conform version ranges."""
     parameters = locals()
     parameters.pop("click_ctx")
 
+    if pipeline and prescription:
+        sys.exit("Options --pipeline/--prescription are disjoint")
+
     if library_usage:
         if os.path.isfile(library_usage):
             try:
-                library_usage = json.loads(Path(library_usage).read_text())
+                with open(library_usage, "r") as f:
+                    library_usage = json.load(f)
             except Exception:
                 _LOGGER.error("Failed to load library usage file %r", library_usage)
                 raise
         else:
             library_usage = json.loads(library_usage)
 
+        # Show library usage in the final report.
+        parameters["library_usage"] = library_usage
+
     runtime_environment = RuntimeEnvironment.load(runtime_environment)
+    parameters["runtime_environment"] = runtime_environment.to_dict()
+
     decision_type = DecisionType.by_name(decision_type)
     requirements_format = PythonRecommendationOutput.by_name(requirements_format)
     project = _instantiate_project(requirements, runtime_environment=runtime_environment)
-    pipeline_config = None if pipeline is None else PipelineBuilder.load(pipeline)
-
+    parameters["requirements"] = project.pipfile.to_dict()
     parameters["project"] = project.to_dict()
+
+    pipeline_config = None if pipeline is None else PipelineBuilder.load(pipeline)
     if pipeline_config is not None:
         parameters["pipeline"] = pipeline_config.to_dict()
 
+    prescription_instance = None
+    if prescription:
+        if len(prescription) == 1:
+            # Click does not support multiple parameters when supplied via env vars. Perform split on delimiter.
+            prescription_instance = Prescription.load(*prescription[0].split(","))
+        else:
+            prescription_instance = Prescription.load(*prescription)
+
     # Use current time to make sure we have possibly reproducible runs - the seed is reported.
     seed = seed if seed is not None else int(time.time())
     predictor_class = _get_dependency_monkey_predictor(predictor, decision_type)
@@ -737,9 +864,12 @@ def dependency_monkey(
         limit_latest_versions=limit_latest_versions,
         decision_type=decision_type,
         pipeline_config=pipeline_config,
+        prescription=prescription_instance,
         cli_parameters=parameters,
     )
 
+    del prescription  # No longer needed, garbage collect it.
+
     context_content = {}
     try:
         with open(context) as f:
@@ -747,6 +877,7 @@ def dependency_monkey(
     except (FileNotFoundError, IOError):
         # IOError raised if context is too large to be handled with open.
         context_content = json.loads(context)
+    parameters["context"] = context_content
 
     dependency_monkey_runner = DependencyMonkey(
         resolver=resolver,
@@ -774,9 +905,64 @@ def dependency_monkey(
         plot=plot,
         with_devel=dev,
         user_stack_scoring=False,
+        # Keep verbose output (stating pipeline units run) in dependency-monkey.
+        verbose=True,
     )
 
     click_ctx.exit(int(exit_code != 0))
 
 
+@cli.command("validate-prescriptions")
+@click.argument("prescriptions", nargs=1, metavar="PRESCRIPTION_DIR")
+@click.option(
+    "--show-unit-names",
+    envvar="THOTH_ADVISER_VALIDATE_PRESCRIPTION_SHOW_UNIT_NAMES",
+    is_flag=True,
+    default=False,
+    show_default=True,
+    help="Show names of prescriptions that were validated.",
+)
+@click.option(
+    "--output",
+    envvar="THOTH_ADVISER_VALIDATE_PRESCRIPTION_OUTPUT",
+    type=str,
+    metavar="PRESCRIPTIONS.pickle",
+    required=False,
+    help="Serialize validated prescriptions into an output pickle file.",
+)
+def validate_prescription(prescriptions: str, show_unit_names: bool, output: str) -> None:
+    """Validate the given prescription."""
+    _LOGGER.info("Validating prescriptions in %r", prescriptions)
+    prescription = Prescription.validate(prescriptions)
+    _LOGGER.info("Prescriptions %r validated successfully", prescriptions)
+
+    result = {
+        "prescriptions": [{"name": p[0], "release": p[1]} for p in prescription.prescriptions],
+        "count": {
+            "boots_count": sum(1 for _ in prescription.iter_boot_units()),
+            "pseudonyms_count": sum(1 for _ in prescription.iter_pseudonym_units()),
+            "sieves_count": sum(1 for _ in prescription.iter_sieve_units()),
+            "steps_count": sum(1 for _ in prescription.iter_step_units()),
+            "strides_count": sum(1 for _ in prescription.iter_stride_units()),
+            "wraps_count": sum(1 for _ in prescription.iter_wrap_units()),
+        },
+        "count_all": sum(1 for _ in prescription.units),
+    }
+
+    if show_unit_names:
+        result["boots"] = sorted(prescription.boots_dict.keys())
+        result["pseudonyms"] = sorted(prescription.pseudonyms_dict.keys())
+        result["sieves"] = sorted(prescription.sieves_dict.keys())
+        result["steps"] = sorted(prescription.steps_dict.keys())
+        result["strides"] = sorted(prescription.strides_dict.keys())
+        result["wraps"] = sorted(prescription.wraps_dict.keys())
+
+    yaml.safe_dump(result, sys.stdout)
+
+    if output:
+        _LOGGER.info("Writing validated prescriptions to %r", output)
+        with open(output, "wb") as fp:
+            pickle.dump(prescription, fp)
+
+
 __name__ == "__main__" and cli()
