@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # thoth-adviser
-# Copyright(C) 2019, 2020 Fridolin Pokorny
+# Copyright(C) 2019 - 2021 Fridolin Pokorny
 #
 # This program is free software: you can redistribute it and / or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,8 +23,10 @@ import json
 from typing import Any
 from typing import Dict
 from typing import Generator
+from typing import Type
 from typing import List
 from typing import Optional
+from typing import TYPE_CHECKING
 from itertools import chain
 
 import attr
@@ -38,13 +40,17 @@ from .exceptions import InternalError
 from .exceptions import UnknownPipelineUnitError
 from .exceptions import PipelineConfigurationError
 from .pipeline_config import PipelineConfig
-from .pseudonym import Pseudonym
-from .boot import Boot
-from .sieve import Sieve
-from .step import Step
-from .stride import Stride
-from .wrap import Wrap
-from .unit import Unit
+from .prescription import UnitPrescription
+
+if TYPE_CHECKING:
+    from .prescription import Prescription  # noqa: F401
+    from .unit_types import BootType
+    from .unit_types import PseudonymType
+    from .unit_types import SieveType
+    from .unit_types import StepType
+    from .unit_types import StrideType
+    from .unit_types import UnitType
+    from .unit_types import WrapType
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -56,79 +62,83 @@ class PipelineBuilderContext:
     graph = attr.ib(type=GraphDatabase, kw_only=True, default=None)
     project = attr.ib(type=Project, kw_only=True, default=None)
     library_usage = attr.ib(type=Optional[Dict[str, Any]], kw_only=True, default=None)
+    labels = attr.ib(type=Dict[str, str], kw_only=True, default=attr.Factory(dict))
     decision_type = attr.ib(type=Optional[DecisionType], kw_only=True, default=None)
     recommendation_type = attr.ib(type=Optional[RecommendationType], kw_only=True, default=None)
+    cli_parameters = attr.ib(type=Dict[str, Any], kw_only=True, default=attr.Factory(dict))
+    prescription = attr.ib(type=Optional["Prescription"], kw_only=True, default=None)
+    iteration = attr.ib(type=int, kw_only=True, default=0)
 
-    _boots = attr.ib(type=Dict[Optional[str], List[Boot]], factory=dict, kw_only=True)
-    _pseudonyms = attr.ib(type=Dict[str, List[Pseudonym]], factory=dict, kw_only=True)
-    _sieves = attr.ib(type=Dict[Optional[str], List[Sieve]], factory=dict, kw_only=True)
-    _steps = attr.ib(type=Dict[Optional[str], List[Step]], factory=dict, kw_only=True)
-    _strides = attr.ib(type=Dict[Optional[str], List[Stride]], factory=dict, kw_only=True)
-    _wraps = attr.ib(type=Dict[Optional[str], List[Wrap]], factory=dict, kw_only=True)
-    _boots_included = attr.ib(type=Dict[str, List[Boot]], factory=dict, kw_only=True)
-    _pseudonyms_included = attr.ib(type=Dict[str, List[Pseudonym]], factory=dict, kw_only=True)
-    _sieves_included = attr.ib(type=Dict[str, List[Sieve]], factory=dict, kw_only=True)
-    _steps_included = attr.ib(type=Dict[str, List[Step]], factory=dict, kw_only=True)
-    _strides_included = attr.ib(type=Dict[str, List[Stride]], factory=dict, kw_only=True)
-    _wraps_included = attr.ib(type=Dict[str, List[Wrap]], factory=dict, kw_only=True)
+    _boots = attr.ib(type=Dict[Optional[str], List["BootType"]], factory=dict, kw_only=True)
+    _pseudonyms = attr.ib(type=Dict[str, List["PseudonymType"]], factory=dict, kw_only=True)
+    _sieves = attr.ib(type=Dict[Optional[str], List["SieveType"]], factory=dict, kw_only=True)
+    _steps = attr.ib(type=Dict[Optional[str], List["StepType"]], factory=dict, kw_only=True)
+    _strides = attr.ib(type=Dict[Optional[str], List["StrideType"]], factory=dict, kw_only=True)
+    _wraps = attr.ib(type=Dict[Optional[str], List["WrapType"]], factory=dict, kw_only=True)
+    _boots_included = attr.ib(type=Dict[str, List["BootType"]], factory=dict, kw_only=True)
+    _pseudonyms_included = attr.ib(type=Dict[str, List["PseudonymType"]], factory=dict, kw_only=True)
+    _sieves_included = attr.ib(type=Dict[str, List["SieveType"]], factory=dict, kw_only=True)
+    _steps_included = attr.ib(type=Dict[str, List["StepType"]], factory=dict, kw_only=True)
+    _strides_included = attr.ib(type=Dict[str, List["StrideType"]], factory=dict, kw_only=True)
+    _wraps_included = attr.ib(type=Dict[str, List["WrapType"]], factory=dict, kw_only=True)
 
     @property
-    def boots(self) -> List[Boot]:
+    def boots(self) -> List["BootType"]:
         """Get all boots registered to this pipeline builder context."""
         return list(chain(*self._boots.values()))
 
     @property
-    def boots_dict(self) -> Dict[Optional[str], List[Boot]]:
+    def boots_dict(self) -> Dict[Optional[str], List["BootType"]]:
         """Get boots as a dictionary mapping."""
         return self._boots
 
     @property
-    def pseudonyms(self) -> List[Pseudonym]:
+    def pseudonyms(self) -> List["PseudonymType"]:
         """Get all pseudonyms registered to this pipeline builder context."""
         return list(chain(*self._pseudonyms.values()))
 
     @property
-    def pseudonyms_dict(self) -> Dict[str, List[Pseudonym]]:
+    def pseudonyms_dict(self) -> Dict[str, List["PseudonymType"]]:
         """Get pseudonyms as a dictionary mapping."""
         return self._pseudonyms
 
     @property
-    def sieves(self) -> List[Sieve]:
+    def sieves(self) -> List["SieveType"]:
         """Get all sieves registered to this pipeline builder context."""
         return list(chain(*self._sieves.values()))
 
     @property
-    def sieves_dict(self) -> Dict[Optional[str], List[Sieve]]:
+    def sieves_dict(self) -> Dict[Optional[str], List["SieveType"]]:
         """Get sieves as a dictionary mapping."""
         return self._sieves
 
     @property
-    def steps(self) -> List[Step]:
+    def steps(self) -> List["StepType"]:
         """Get all steps registered to this pipeline builder context."""
         return list(chain(*self._steps.values()))
 
     @property
-    def steps_dict(self) -> Dict[Optional[str], List[Step]]:
+    def steps_dict(self) -> Dict[Optional[str], List["StepType"]]:
         """Get steps as a dictionary mapping."""
         return self._steps
 
     @property
-    def strides(self) -> List[Stride]:
+    def strides(self) -> List["StrideType"]:
         """Get all strides registered to this pipeline builder context."""
         return list(chain(*self._strides.values()))
 
     @property
-    def strides_dict(self) -> Dict[Optional[str], List[Stride]]:
+    def strides_dict(self) -> Dict[Optional[str], List["StrideType"]]:
         """Get strides as a dictionary mapping."""
         return self._strides
 
     @property
-    def wraps(self) -> List[Wrap]:
+    def wraps(self) -> List["WrapType"]:
         """Get all wraps registered to this pipeline builder context."""
         return list(chain(*self._wraps.values()))
 
     @property
-    def wraps_dict(self) -> Dict[Optional[str], List[Wrap]]:
+    def wraps_dict(self) -> Dict[Optional[str], List["WrapType"]]:
         """Get wraps as a dictionary mapping."""
         return self._wraps
 
@@ -140,52 +150,76 @@ class PipelineBuilderContext:
         if self.decision_type is None and self.recommendation_type is None:
             raise ValueError("Cannot instantiate builder context not specific to adviser nor dependency monkey")
 
-    def is_included(self, unit_class: type) -> bool:
+    def is_included(self, unit_class: Type["UnitType"]) -> bool:
         """Check if the given pipeline unit is already included in the pipeline configuration."""
-        if issubclass(unit_class, Boot):
-            return unit_class.__name__ in self._boots_included
-        elif issubclass(unit_class, Pseudonym):
-            return unit_class.__name__ in self._pseudonyms_included
-        elif issubclass(unit_class, Sieve):
-            return unit_class.__name__ in self._sieves_included
-        elif issubclass(unit_class, Step):
-            return unit_class.__name__ in self._steps_included
-        elif issubclass(unit_class, Stride):
-            return unit_class.__name__ in self._strides_included
-        elif issubclass(unit_class, Wrap):
-            return unit_class.__name__ in self._wraps_included
+        if unit_class.is_boot_unit_type():
+            return unit_class.get_unit_name() in self._boots_included
+        elif unit_class.is_pseudonym_unit_type():
+            return unit_class.get_unit_name() in self._pseudonyms_included
+        elif unit_class.is_sieve_unit_type():
+            return unit_class.get_unit_name() in self._sieves_included
+        elif unit_class.is_step_unit_type():
+            return unit_class.get_unit_name() in self._steps_included
+        elif unit_class.is_stride_unit_type():
+            return unit_class.get_unit_name() in self._strides_included
+        elif unit_class.is_wrap_unit_type():
+            return unit_class.get_unit_name() in self._wraps_included
 
-        raise InternalError(f"Unknown unit {unit_class.__name__!r}")
+        raise InternalError(f"Unknown unit {unit_class.get_unit_name()!r} of type {unit_class}")
 
-    def get_included_boots(self, boot_class: type) -> List[Boot]:
+    def get_included_boots(self, boot_class: Type["UnitType"]) -> Generator["BootType", None, None]:
         """Get included boots of the provided boot class."""
-        assert issubclass(boot_class, Boot)
-        return self._boots_included.get(boot_class.__name__, [])
+        assert boot_class.is_boot_unit_type()
+        yield from self._boots_included.get(boot_class.get_unit_name(), [])
 
-    def get_included_pseudonyms(self, pseudonym_class: type) -> List[Pseudonym]:
+    def get_included_boot_names(self) -> Generator[str, None, None]:
+        """Get names of included boots."""
+        yield from self._boots_included.keys()
+
+    def get_included_pseudonyms(self, pseudonym_class: Type["PseudonymType"]) -> Generator["PseudonymType", None, None]:
         """Get included sieves of the provided sieve class."""
-        assert issubclass(pseudonym_class, Pseudonym)
-        return self._pseudonyms_included.get(pseudonym_class.__name__, [])
+        assert pseudonym_class.is_pseudonym_unit_type()
+        yield from self._pseudonyms_included.get(pseudonym_class.get_unit_name(), [])
 
-    def get_included_sieves(self, sieve_class: type) -> List[Sieve]:
+    def get_included_pseudonym_names(self) -> Generator[str, None, None]:
+        """Get names of included pseudonyms."""
+        yield from self._pseudonyms_included.keys()
+
+    def get_included_sieves(self, sieve_class: Type["SieveType"]) -> Generator["SieveType", None, None]:
         """Get included sieves of the provided sieve class."""
-        assert issubclass(sieve_class, Sieve)
-        return self._sieves_included.get(sieve_class.__name__, [])
+        assert sieve_class.is_sieve_unit_type()
+        yield from self._sieves_included.get(sieve_class.get_unit_name(), [])
 
-    def get_included_steps(self, step_class: type) -> List[Step]:
+    def get_included_sieve_names(self) -> Generator[str, None, None]:
+        """Get names of included sieves."""
+        yield from self._sieves_included.keys()
+
+    def get_included_steps(self, step_class: Type["StepType"]) -> Generator["StepType", None, None]:
         """Get included steps of the provided step class."""
-        assert issubclass(step_class, Step)
-        return self._steps_included.get(step_class.__name__, [])
+        assert step_class.is_step_unit_type()
+        yield from self._steps_included.get(step_class.get_unit_name(), [])
 
-    def get_included_strides(self, stride_class: type) -> List[Stride]:
+    def get_included_step_names(self) -> Generator[str, None, None]:
+        """Get names of included steps."""
+        yield from self._steps_included.keys()
+
+    def get_included_strides(self, stride_class: Type["StrideType"]) -> Generator["StrideType", None, None]:
         """Get included strides of the provided stride class."""
-        assert issubclass(stride_class, Stride)
-        return self._strides_included.get(stride_class.__name__, [])
+        assert stride_class.is_stride_unit_type()
+        yield from self._strides_included.get(stride_class.get_unit_name(), [])
 
-    def get_included_wraps(self, wrap_class: type) -> List[Wrap]:
+    def get_included_stride_names(self) -> Generator[str, None, None]:
+        """Get names of included strides."""
+        yield from self._strides_included.keys()
+
+    def get_included_wraps(self, wrap_class: Type["WrapType"]) -> Generator["WrapType", None, None]:
         """Get included wraps of the provided wrap class."""
-        assert issubclass(wrap_class, Wrap)
-        return self._wraps_included.get(wrap_class.__name__, [])
+        assert wrap_class.is_wrap_unit_type()
+        yield from self._wraps_included.get(wrap_class.get_unit_name(), [])
+
+    def get_included_wrap_names(self) -> Generator[str, None, None]:
+        """Get names of included wraps."""
+        yield from self._wraps_included.keys()
 
     def is_adviser_pipeline(self) -> bool:
         """Check if the pipeline built is meant for adviser."""
@@ -195,41 +229,41 @@ class PipelineBuilderContext:
         """Check if the pipeline built is meant for Dependency Monkey."""
         return self.decision_type is not None and self.recommendation_type is None
 
-    def add_unit(self, unit: Unit) -> None:
+    def add_unit(self, unit: "UnitType") -> None:
         """Add the given unit to pipeline configuration."""
         package_name: Optional[str] = unit.configuration.get("package_name")
 
-        if isinstance(unit, Boot):
-            self._boots_included.setdefault(unit.__class__.__name__, []).append(unit)
+        if unit.is_boot_unit_type():
+            self._boots_included.setdefault(unit.name, []).append(unit)
             self._boots.setdefault(package_name, []).append(unit)
             return
-        elif isinstance(unit, Pseudonym):
+        elif unit.is_pseudonym_unit_type():
             if not package_name:
                 raise PipelineConfigurationError(
-                    f"Pipeline cannot be constructed as unit {unit.__class__.__name__!r} of type Pseudonym "
+                    f"Pipeline cannot be constructed as unit {unit.name!r} of type Pseudonym "
                     f"did not provide any package name configuration: {unit.configuration!r}"
                 )
-            self._pseudonyms_included.setdefault(unit.__class__.__name__, []).append(unit)
+            self._pseudonyms_included.setdefault(unit.name, []).append(unit)
             self._pseudonyms.setdefault(package_name, []).append(unit)
             return
-        elif isinstance(unit, Sieve):
-            self._sieves_included.setdefault(unit.__class__.__name__, []).append(unit)
+        elif unit.is_sieve_unit_type():
+            self._sieves_included.setdefault(unit.name, []).append(unit)
             self._sieves.setdefault(package_name, []).append(unit)
             return
-        elif isinstance(unit, Step):
-            self._steps_included.setdefault(unit.__class__.__name__, []).append(unit)
+        elif unit.is_step_unit_type():
+            self._steps_included.setdefault(unit.name, []).append(unit)
             self._steps.setdefault(package_name, []).append(unit)
             return
-        elif isinstance(unit, Stride):
-            self._strides_included.setdefault(unit.__class__.__name__, []).append(unit)
+        elif unit.is_stride_unit_type():
+            self._strides_included.setdefault(unit.name, []).append(unit)
             self._strides.setdefault(package_name, []).append(unit)
             return
-        elif isinstance(unit, Wrap):
-            self._wraps_included.setdefault(unit.__class__.__name__, []).append(unit)
+        elif unit.is_wrap_unit_type():
+            self._wraps_included.setdefault(unit.name, []).append(unit)
             self._wraps.setdefault(package_name, []).append(unit)
             return
 
-        raise InternalError(f"Unknown unit {unit!r} of type {unit.__class__.__name__!r}")
+        raise InternalError(f"Unknown unit {unit!r} of type {unit.name!r}")
 
 
 class PipelineBuilder:
@@ -242,7 +276,7 @@ class PipelineBuilder:
         raise NotImplementedError("Cannot instantiate pipeline builder")
 
     @staticmethod
-    def _iter_units() -> Generator[type, None, None]:
+    def _iter_units(ctx: PipelineBuilderContext) -> Generator["UnitType", None, None]:
         """Iterate over pipeline units available in this implementation."""
         # Imports placed here to simplify tests.
         import thoth.adviser.boots
@@ -254,21 +288,33 @@ class PipelineBuilder:
 
         for boot_name in thoth.adviser.boots.__all__:
             yield getattr(thoth.adviser.boots, boot_name)
+        if ctx.prescription:
+            yield from ctx.prescription.iter_boot_units()
 
         for pseudonym_name in thoth.adviser.pseudonyms.__all__:
             yield getattr(thoth.adviser.pseudonyms, pseudonym_name)
+        if ctx.prescription:
+            yield from ctx.prescription.iter_pseudonym_units()
 
         for sieve_name in thoth.adviser.sieves.__all__:
             yield getattr(thoth.adviser.sieves, sieve_name)
+        if ctx.prescription:
+            yield from ctx.prescription.iter_sieve_units()
 
         for step_name in thoth.adviser.steps.__all__:
             yield getattr(thoth.adviser.steps, step_name)
+        if ctx.prescription:
+            yield from ctx.prescription.iter_step_units()
 
         for stride_name in thoth.adviser.strides.__all__:
             yield getattr(thoth.adviser.strides, stride_name)
+        if ctx.prescription:
+            yield from ctx.prescription.iter_stride_units()
 
         for wrap_name in thoth.adviser.wraps.__all__:
             yield getattr(thoth.adviser.wraps, wrap_name)
+        if ctx.prescription:
+            yield from ctx.prescription.iter_wrap_units()
 
     @classmethod
     def _build_configuration(cls, ctx: PipelineBuilderContext) -> PipelineConfig:
@@ -284,43 +330,50 @@ class PipelineBuilder:
         )
 
         change = True
-        while change:
-            change = False
-            for unit_class in cls._iter_units():
-                if unit_class.__name__ in blocked_units:
-                    _LOGGER.debug(
-                        "Avoiding adding pipeline unit %r based on blocked units configuration",
-                        unit_class.__name__,
-                    )
-                    continue
+        ctx.iteration = -1
+        try:
+            while change:
+                change = False
+                ctx.iteration += 1
+                for unit_class in cls._iter_units(ctx):
+                    unit_name = unit_class.get_unit_name()
+                    if unit_name in blocked_units:
+                        _LOGGER.debug(
+                            "Avoiding adding pipeline unit %r based on blocked units configuration",
+                            unit_name,
+                        )
+                        continue
 
-                unit_configuration = unit_class.should_include(ctx)  # type: ignore
-                if unit_configuration is None:
-                    _LOGGER.debug(
-                        "Pipeline unit %r will not be included in the pipeline configuration in this round",
-                        unit_class.__name__,
-                    )
-                    continue
+                    for unit_configuration in unit_class.should_include(ctx):
+                        if unit_configuration is None:
+                            _LOGGER.debug(
+                                "Pipeline unit %r will not be included in the pipeline configuration in this round",
+                                unit_name,
+                            )
+                            continue
 
-                change = True
+                        change = True
 
-                _LOGGER.debug(
-                    "Including pipeline unit %r in pipeline configuration with unit configuration %r",
-                    unit_class.__name__,
-                    unit_configuration,
-                )
-                unit_instance = unit_class()
+                        _LOGGER.debug(
+                            "Including pipeline unit %r in pipeline configuration with unit configuration %r",
+                            unit_name,
+                            unit_configuration,
+                        )
+                        unit_instance = unit_class()  # type: ignore
 
-                # Always perform update, even with an empty dict. Update triggers a schema check.
-                try:
-                    unit_instance.update_configuration(unit_configuration)
-                except Exception as exc:
-                    raise PipelineConfigurationError(
-                        f"Filed to initialize pipeline unit configuration for {unit_class.__name__!r} "
-                        f"with configuration {unit_configuration!r}: {str(exc)}"
-                    ) from exc
+                        # Always perform update, even with an empty dict. Update triggers a schema check.
+                        try:
+                            unit_instance.update_configuration(unit_configuration)
+                        except Exception as exc:
+                            raise PipelineConfigurationError(
+                                f"Filed to initialize pipeline unit configuration for {unit_name!r} "
+                                f"with configuration {unit_configuration!r}: {str(exc)}"
+                            ) from exc
 
-                ctx.add_unit(unit_instance)
+                        ctx.add_unit(unit_instance)
+        finally:
+            # Once the build pipeline is constructed or fails to construct, we can clear cached results.
+            UnitPrescription.SHOULD_INCLUDE_CACHE.clear()
 
         pipeline = PipelineConfig(
             boots=ctx.boots_dict,
@@ -340,7 +393,7 @@ class PipelineBuilder:
         return pipeline
 
     @staticmethod
-    def _do_instantiate_from_dict(module: object, configuration_entry: Dict[str, Any]) -> Unit:
+    def _do_instantiate_from_dict(module: object, configuration_entry: Dict[str, Any]) -> "UnitType":
         """Instantiate a pipeline unit from a dict representation."""
         if "name" not in configuration_entry:
             raise ValueError(f"No pipeline unit name provided in the configuration entry: {configuration_entry!r}")
@@ -350,14 +403,14 @@ class PipelineBuilder:
         except AttributeError as exc:
             raise UnknownPipelineUnitError(f"Cannot import unit {configuration_entry['name']}: {str(exc)}") from exc
 
-        unit: Unit = unit_class()
+        unit: "UnitType" = unit_class()
 
         if configuration_entry.get("configuration"):
             try:
                 unit.update_configuration(configuration_entry["configuration"])
             except Exception as exc:
                 raise PipelineConfigurationError(
-                    f"Filed to initialize pipeline unit configuration for {unit_class.__name__!r} "
+                    f"Filed to initialize pipeline unit configuration for {unit.name!r} "
                     f"with configuration {configuration_entry['configuration']!r}: {str(exc)}"
                 ) from exc
 
@@ -374,46 +427,46 @@ class PipelineBuilder:
         import thoth.adviser.strides
         import thoth.adviser.wraps
 
-        boots: Dict[Optional[str], List[Boot]] = {}
+        boots: Dict[Optional[str], List["BootType"]] = {}
         for boot_entry in dict_.pop("boots", []) or []:
-            boot_unit: Boot = cls._do_instantiate_from_dict(thoth.adviser.boots, boot_entry)  # type: ignore
+            boot_unit: "BootType" = cls._do_instantiate_from_dict(thoth.adviser.boots, boot_entry)
             package_name = boot_unit.configuration.get("package_name")
             boots.setdefault(package_name, []).append(boot_unit)
 
-        pseudonyms: Dict[str, List[Pseudonym]] = {}
+        pseudonyms: Dict[str, List["PseudonymType"]] = {}
         for pseudonym_entry in dict_.pop("pseudonyms", []) or []:
-            unit: Pseudonym = cls._do_instantiate_from_dict(thoth.adviser.pseudonyms, pseudonym_entry)  # type: ignore
+            unit: "PseudonymType" = cls._do_instantiate_from_dict(thoth.adviser.pseudonyms, pseudonym_entry)
 
             package_name = unit.configuration.get("package_name")
             if not package_name:
                 raise PipelineConfigurationError(
-                    f"Pipeline cannot be constructed as unit {unit.__class__.__name__!r} of type Pseudonym "
+                    f"Pipeline cannot be constructed as unit {unit.name!r} of type Pseudonym "
                     f"did not provide any package name configuration: {unit.configuration!r}"
                 )
 
             pseudonyms.setdefault(package_name, []).append(unit)
 
-        sieves: Dict[Optional[str], List[Sieve]] = {}
+        sieves: Dict[Optional[str], List["SieveType"]] = {}
         for sieve_entry in dict_.pop("sieves", []) or []:
-            sieve_unit: Sieve = cls._do_instantiate_from_dict(thoth.adviser.sieves, sieve_entry)  # type: ignore
+            sieve_unit: "SieveType" = cls._do_instantiate_from_dict(thoth.adviser.sieves, sieve_entry)
             package_name = sieve_unit.configuration.get("package_name")
             sieves.setdefault(package_name, []).append(sieve_unit)
 
-        steps: Dict[Optional[str], List[Step]] = {}
+        steps: Dict[Optional[str], List["StepType"]] = {}
         for step_entry in dict_.pop("steps", []) or []:
-            step_unit: Step = cls._do_instantiate_from_dict(thoth.adviser.steps, step_entry)  # type: ignore
+            step_unit: "StepType" = cls._do_instantiate_from_dict(thoth.adviser.steps, step_entry)
             package_name = step_unit.configuration.get("package_name")
             steps.setdefault(package_name, []).append(step_unit)
 
-        strides: Dict[Optional[str], List[Stride]] = {}
+        strides: Dict[Optional[str], List["StrideType"]] = {}
         for stride_entry in dict_.pop("strides", []) or []:
-            stride_unit: Stride = cls._do_instantiate_from_dict(thoth.adviser.strides, stride_entry)  # type: ignore
+            stride_unit: "StrideType" = cls._do_instantiate_from_dict(thoth.adviser.strides, stride_entry)
             package_name = stride_unit.configuration.get("package_name")
             strides.setdefault(package_name, []).append(stride_unit)
 
-        wraps: Dict[Optional[str], List[Wrap]] = {}
+        wraps: Dict[Optional[str], List["WrapType"]] = {}
         for wrap_entry in dict_.pop("wraps", []) or []:
-            wrap_unit: Wrap = cls._do_instantiate_from_dict(thoth.adviser.wraps, wrap_entry)  # type: ignore
+            wrap_unit: "WrapType" = cls._do_instantiate_from_dict(thoth.adviser.wraps, wrap_entry)
             package_name = wrap_unit.configuration.get("package_name")
             wraps.setdefault(package_name, []).append(wrap_unit)
 
@@ -451,15 +504,21 @@ class PipelineBuilder:
         recommendation_type: RecommendationType,
         graph: GraphDatabase,
         project: Project,
+        labels: Dict[str, str],
         library_usage: Optional[Dict[str, Any]],
+        prescription: Optional["Prescription"],
+        cli_parameters: Dict[str, Any],
     ) -> PipelineConfig:
         """Get adviser's pipeline configuration."""
         return cls._build_configuration(
             PipelineBuilderContext(
                 graph=graph,
                 project=project,
+                labels=labels,
                 library_usage=library_usage,
                 recommendation_type=recommendation_type,
+                prescription=prescription,
+                cli_parameters=cli_parameters,
             )
         )
 
@@ -470,14 +529,20 @@ class PipelineBuilder:
         decision_type: DecisionType,
         graph: GraphDatabase,
         project: Project,
+        labels: Dict[str, str],
         library_usage: Optional[Dict[str, Any]],
+        prescription: Optional["Prescription"],
+        cli_parameters: Dict[str, Any],
     ) -> PipelineConfig:
         """Get dependency-monkey's pipeline configuration."""
         return cls._build_configuration(
             PipelineBuilderContext(
                 graph=graph,
                 project=project,
+                labels=labels,
                 library_usage=library_usage,
                 decision_type=decision_type,
+                prescription=prescription,
+                cli_parameters=cli_parameters,
             )
         )
