@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # thoth-adviser
-# Copyright(C) 2019, 2020 Fridolin Pokorny
+# Copyright(C) 2019 - 2021 Fridolin Pokorny
 #
 # This program is free software: you can redistribute it and / or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,12 +29,15 @@ from typing import List
 from typing import Union
 from typing import Set
 from typing import Iterator
+from typing import TYPE_CHECKING
 import logging
 from itertools import chain
 import contextlib
 import signal
 import weakref
+import heapq
 
+import attr
 import matplotlib
 import matplotlib.pyplot as plt
 from matplotlib.font_manager import FontProperties
@@ -71,7 +74,9 @@ from .state import State
 from .unit import Unit
 from .utils import log_once
 
-import attr
+if TYPE_CHECKING:
+    from .prescription import Prescription  # noqa: F401
+
 
 _LOGGER = logging.getLogger(__name__)
 _NO_EXTRAS = frozenset([None])
@@ -155,6 +160,7 @@ class Resolver:
     library_usage = attr.ib(type=Dict[str, Any], kw_only=True, converter=_library_usage)
     graph = attr.ib(type=GraphDatabase, kw_only=True)
     predictor = attr.ib(type=Predictor, kw_only=True)
+    labels = attr.ib(type=Dict[str, str], kw_only=True, default=attr.Factory(dict))
     recommendation_type = attr.ib(type=Optional[RecommendationType], kw_only=True, default=None)
     decision_type = attr.ib(type=Optional[DecisionType], kw_only=True, default=None)
     limit = attr.ib(type=int, kw_only=True, default=DEFAULT_LIMIT)
@@ -172,6 +178,7 @@ class Resolver:
         converter=_limit_latest_versions,  # type: ignore
     )
 
+    prescription = attr.ib(type=Optional["Prescription"], default=None, kw_only=True)
     cli_parameters = attr.ib(type=Dict[str, Any], default=attr.Factory(dict), kw_only=True)
     stop_resolving = attr.ib(type=bool, default=False, kw_only=True)
     log_iteration = attr.ib(type=int, kw_only=True, default=int(os.getenv("THOTH_ADVISER_LOG_ITERATION", 7500)))
@@ -238,12 +245,14 @@ class Resolver:
         self._context = Context(
             project=self.project,
             graph=self.graph,
+            labels=self.labels,
             library_usage=self.library_usage,
             limit=self.limit,
             count=self.count,
             beam=self.beam,
             recommendation_type=self.recommendation_type,
             decision_type=self.decision_type,
+            prescription=self.prescription,
             cli_parameters=self.cli_parameters,
         )
 
@@ -254,21 +263,15 @@ class Resolver:
             package_boots.extend(self.pipeline.boots_dict.get(package_version.name, []))
 
         for boot in chain(package_boots, self.pipeline.boots_dict.get(None, [])):
-            _LOGGER.debug("Running boot %r", boot.__class__.__name__)
+            _LOGGER.debug("Running boot %r", boot.name)
             boot.unit_run = True
             try:
                 boot.run()
             except NotAcceptable as exc:
-                msg = f"Boot pipeline unit {boot.__class__.__name__!r} failed: {str(exc)}"
-                self.context.stack_info.append(
-                    {
-                        "message": msg,
-                        "type": "ERROR",
-                    }
-                )
+                msg = f"Boot pipeline unit {boot.name} failed: {str(exc)}"
                 raise CannotProduceStack(msg, stack_info=self.context.stack_info)
             except Exception as exc:
-                raise BootError(f"Failed to run pipeline boot {boot.__class__.__name__!r}: {str(exc)}") from exc
+                raise BootError(f"Failed to run pipeline boot {boot.name!r}: {str(exc)}") from exc
 
     def _run_sieves(
         self, package_versions: List[PackageVersion], *, log_level: int = logging.DEBUG
@@ -279,7 +282,7 @@ class Resolver:
             for sieve in chain(
                 self.pipeline.sieves_dict.get(package_versions[0].name, []), self.pipeline.sieves_dict.get(None, [])
             ):
-                _LOGGER.debug("Running sieve %r", sieve.__class__.__name__)
+                _LOGGER.debug("Running sieve %r", sieve.name)
                 sieve.unit_run = True
                 try:
                     result = sieve.run(result)
@@ -289,14 +292,14 @@ class Resolver:
                     _LOGGER.log(
                         log_level,
                         "Sieve %r removed packages %r: %s",
-                        sieve.__class__.__name__,
-                        str(exc),
+                        sieve.name,
+                        exc,
                     )
                     result = []  # type: ignore
                     break
                 except Exception as exc:
                     raise SieveError(
-                        f"Failed to run sieve {sieve.__class__.__name__!r} for "
+                        f"Failed to run sieve {sieve.name!r} for "
                         f"Python packages {[pv.to_tuple() for pv in package_versions]}: {str(exc)}"
                     ) from exc
 
@@ -335,17 +338,18 @@ class Resolver:
         score_addition = 0.0
         justification_addition = []
         skip_package = False
+        step_result = None
         for step in chain(
             self.pipeline.steps_dict.get(package_version.name, []), self.pipeline.steps_dict.get(None, [])
         ):
-            _LOGGER.debug("Running step %r for %r", step.__class__.__name__, package_version_tuple)
+            _LOGGER.debug("Running step %r for %r", step.name, package_version_tuple)
             step.unit_run = True
 
             if multi_package_resolution and not step.configuration["multi_package_resolution"]:
                 _LOGGER.debug(
                     "Skipping running step %r - this step was already run for package %r "
                     "and the given step has no multi_package_resolution flag set",
-                    step.__class__.__name__,
+                    step.name,
                     package_version_tuple,
                 )
                 continue
@@ -359,9 +363,9 @@ class Resolver:
                     self._log_step_skip_package,
                     package_version_tuple,
                     "Step %r discarded addition of package %r: %s",
-                    step.__class__.__name__,
+                    step.name,
                     package_version_tuple,
-                    str(exc),
+                    exc,
                     level=log_level,
                 )
                 skip_package = True
@@ -372,9 +376,9 @@ class Resolver:
                     self._log_step_not_acceptable,
                     package_version_tuple,
                     "Step %r discarded addition of package %r: %s",
-                    step.__class__.__name__,
+                    step.name,
                     package_version_tuple,
-                    str(exc),
+                    exc,
                     level=log_level,
                 )
                 if not user_stack_scoring:
@@ -385,8 +389,7 @@ class Resolver:
                 return None
             except Exception as exc:
                 raise StepError(
-                    f"Failed to run step {step.__class__.__name__!r} for Python package "
-                    f"{package_version_tuple!r}: {str(exc)}"
+                    f"Failed to run step {step.name!r} for Python package {package_version_tuple!r}: {str(exc)}"
                 ) from exc
 
             if step_result:
@@ -395,18 +398,18 @@ class Resolver:
                 if step_score_addition is not None:
                     if math.isnan(step_score_addition):
                         raise StepError(
-                            f"Step {step.__class__.__name__} returned score which is not a number",
+                            f"Step {step.name} returned score which is not a number",
                         )
 
                     if math.isinf(step_score_addition):
                         raise StepError(
-                            f"Step {step.__class__.__name__} returned score that is infinite",
+                            f"Step {step.name} returned score that is infinite",
                         )
 
                     if step_score_addition > step.SCORE_MAX:
                         _LOGGER.warning(
                             "Step %r returned score higher than allowed (%g), normalizing to %g",
-                            step.__class__.__name__,
+                            step.name,
                             step_score_addition,
                             step.SCORE_MAX,
                         )
@@ -414,7 +417,7 @@ class Resolver:
                     elif step_score_addition < step.SCORE_MIN:
                         _LOGGER.warning(
                             "Step %r returned score lower than allowed (%g), normalizing to %g",
-                            step.__class__.__name__,
+                            step.name,
                             step_score_addition,
                             step.SCORE_MIN,
                         )
@@ -436,27 +439,27 @@ class Resolver:
             # Optimization - reuse the old one as it would be discarded anyway.
             cloned_state = state
             if not user_stack_scoring and (
-                (not skip_package and score_addition != 0.0)
-                or (not state.unresolved_dependencies and not unresolved_dependencies)
+                (not skip_package and step_result)
+                or (not state.unresolved_dependencies and (skip_package or not unresolved_dependencies))
             ):
                 self.beam.remove(cloned_state)
 
         if unresolved_dependencies and not skip_package:
             cloned_state.set_unresolved_dependencies(unresolved_dependencies)
-            self._run_pseudonyms(cloned_state, newly_added)
 
-        cloned_state.remove_unresolved_dependency_subtree(package_version_tuple[0])
         if not skip_package:
-            cloned_state.add_justification(justification_addition)
+            cloned_state.remove_unresolved_dependency_subtree(package_version_tuple[0])
             cloned_state.add_resolved_dependency(package_version_tuple)
+            cloned_state.add_justification(justification_addition)
             cloned_state.score += score_addition
+            self._run_pseudonyms(cloned_state, newly_added)
 
         cloned_state.iteration = self.context.iteration
 
         if not user_stack_scoring:
             if cloned_state.unresolved_dependencies:
                 self.predictor.set_reward_signal(cloned_state, package_version_tuple, score_addition)
-                if state is not cloned_state or score_addition != 0.0:
+                if not skip_package and (state is not cloned_state or step_result):
                     self.beam.add_state(cloned_state)
             else:
                 self.predictor.set_reward_signal(cloned_state, package_version_tuple, math.inf)
@@ -470,20 +473,20 @@ class Resolver:
             package_strides.extend(self.pipeline.strides_dict.get(package_name, []))
 
         for stride in chain(package_strides, self.pipeline.strides_dict.get(None, [])):
-            _LOGGER.debug("Running stride %r", stride.__class__.__name__)
+            _LOGGER.debug("Running stride %r", stride.name)
             stride.unit_run = True
             try:
                 stride.run(state)
             except NotAcceptable as exc:
                 _LOGGER.debug(
                     "Stride %r removed final state %r: %s",
-                    stride.__class__.__name__,
+                    stride.name,
                     state,
-                    str(exc),
+                    exc,
                 )
                 return False
             except Exception as exc:
-                raise StrideError(f"Failed to run stride {stride.__class__.__name__!r}: {str(exc)}") from exc
+                raise StrideError(f"Failed to run stride {stride.name!r}: {str(exc)}") from exc
 
         return True
 
@@ -494,12 +497,12 @@ class Resolver:
             package_wraps.extend(self.pipeline.wraps_dict.get(package_name, []))
 
         for wrap in chain(package_wraps, self.pipeline.wraps_dict.get(None, [])):
-            _LOGGER.debug("Running wrap %r", wrap.__class__.__name__)
+            _LOGGER.debug("Running wrap %r", wrap.name)
             wrap.unit_run = True
             try:
                 wrap.run(state)
             except Exception as exc:
-                raise WrapError(f"Failed to run wrap {wrap.__class__.__name__!r} on a final state: {str(exc)}") from exc
+                raise WrapError(f"Failed to run wrap {wrap.name!r} on a final state: {str(exc)}") from exc
 
     def _prepare_user_lock_file(self, *, with_devel: bool = True) -> None:
         """Perform operations on the user's lock file required before running the pipeline.
@@ -585,7 +588,7 @@ class Resolver:
                 package_version = list(self._run_sieves([package_version], log_level=logging.INFO))
             except SkipPackage as exc:
                 skipped_packages.append(package_version)
-                _LOGGER.debug("Package %r skipped by sieves: %s", package_version.name, str(exc))
+                _LOGGER.debug("Package %r skipped by sieves: %s", package_version.name, exc)
                 continue
 
             if not package_version:
@@ -620,13 +623,16 @@ class Resolver:
                 {
                     "type": "INFO",
                     "message": "Score of the supplied lock file is the highest possible "
-                    "according to the current knowledge in Thoth and the parameters used to solve the stack.",
+                    "according to the current knowledge in Thoth and the parameters used to solve the stack",
                     "link": jl("user_stack"),
-                }
+                },
+                {
+                    "type": "WARNING",
+                    "message": "Correctness and the dependency graph of the supplied lock file is not verified",
+                    "link": jl("user_stack"),
+                },
             ]
         )
-        self._run_wraps(state)
-        self.beam.add_state(state)
         _LOGGER.info("User's software stack has a score of %.2f - see %s", state.score, jl("user_stack"))
         return state
 
@@ -681,9 +687,13 @@ class Resolver:
             )
 
         if unresolved:
-            error_msg = "Resolver failed as it was unable to resolve direct dependencies"
-            self.context.stack_info.append({"message": error_msg, "type": "ERROR", "link": jl("unresolved")})
-            _LOGGER.warning("%s - see %s", error_msg, jl("unresolved"))
+            for dep in unresolved:
+                error_msg = f"Resolver failed as it was unable to resolve direct dependency {dep!r}"
+                self.context.stack_info.append(
+                    {"message": error_msg, "type": "ERROR", "link": jl("unresolved"), "package_name": dep}
+                )
+                _LOGGER.warning("%s - see %s", error_msg, jl("unresolved"))
+
             raise UnresolvedDependencies(
                 "Unable to resolve all direct dependencies", unresolved=unresolved, stack_info=self.context.stack_info
             )
@@ -719,18 +729,19 @@ class Resolver:
                 package_versions = list(self._run_sieves(package_versions))
             except SkipPackage as exc:
                 skipped_packages.append(direct_dependency_name)
-                _LOGGER.debug("Package %r skipped by sieves: %s", direct_dependency_name, str(exc))
+                _LOGGER.debug("Package %r skipped by sieves: %s", direct_dependency_name, exc)
                 continue
 
             if not package_versions:
                 msg = (
-                    f"Cannot satisfy direct dependencies - direct dependencies of type {direct_dependency_name!r} "
+                    f"Cannot satisfy direct dependencies: direct dependencies of type {direct_dependency_name!r} "
                     "were removed by pipeline sieves"
                 )
                 self.context.stack_info.append(
                     {
                         "type": "ERROR",
                         "message": msg,
+                        "link": jl("unresolved"),
                     }
                 )
                 raise CannotProduceStack(msg, stack_info=self.context.stack_info)
@@ -746,6 +757,17 @@ class Resolver:
             # dict during iteration.
             direct_dependencies.pop(direct_dependency_name)
 
+        if not direct_dependencies:
+            msg = "Cannot satisfy direct dependencies: all direct dependencies were filtered by sieves"
+            self.context.stack_info.append(
+                {
+                    "type": "ERROR",
+                    "message": msg,
+                    "link": jl("unresolved"),
+                }
+            )
+            raise CannotProduceStack(msg, stack_info=self.context.stack_info)
+
         # Create an initial state which is made out of all the direct dependencies (kept as unresolved) in
         # resolved versions.
         self.beam.wipe()
@@ -763,12 +785,12 @@ class Resolver:
         """Run pseudonym units the given package tuple."""
         package_version: PackageVersion = self.context.get_package_version(package_tuple)
         for unit in self.pipeline.pseudonyms_dict.get(package_tuple[0], []):
-            _LOGGER.debug("Running pseudonym %r", unit.__class__.__name__)
+            _LOGGER.debug("Running pseudonym %r", unit.name)
             unit.unit_run = True
             for pseudonym_package_tuple in unit.run(package_version):
                 # Pseudonyms introduced do not have dependents (we work on direct dependencies - initial state).
                 if pseudonym_package_tuple[0] in state.resolved_dependencies:
-                    _LOGGER.warning(
+                    _LOGGER.debug(
                         "Pseudonym %r already present in a resolved form, skipping pseudonym creation",
                         pseudonym_package_tuple,
                     )
@@ -1062,7 +1084,7 @@ class Resolver:
             except SkipPackage as exc:
                 # This will probably happen occasionally. Let's maintain a separate list for this.
                 skipped_packages.append(dependency_name)
-                _LOGGER.debug("Package %r skipped by sieves: %s", dependency_name, str(exc))
+                _LOGGER.debug("Package %r skipped by sieves: %s", dependency_name, exc)
                 continue
 
             if not package_versions:
@@ -1115,7 +1137,7 @@ class Resolver:
             try:
                 user_stack = self._maybe_score_user_lock_file(with_devel=with_devel)
             except UserLockFileError as exc:
-                _LOGGER.warning("Failed to score user's lock file: %s", str(exc))
+                _LOGGER.warning("Failed to score user's lock file: %s", exc)
             except Exception:
                 _LOGGER.exception("Failed to score supplied user stack, the error is not fatal")
             else:
@@ -1251,7 +1273,7 @@ class Resolver:
                 yield final_state
                 del final_state
         except EagerStopPipeline as exc:
-            _LOGGER.info("Stopping pipeline eagerly as per request: %s", str(exc))
+            _LOGGER.info("Stopping pipeline eagerly as per request: %s", exc)
 
         duration = time.monotonic() - start_time
         _LOGGER.info(
@@ -1285,17 +1307,21 @@ class Resolver:
 
     def resolve(self, *, with_devel: bool = True, user_stack_scoring: bool = True) -> Report:
         """Resolve software stacks and return resolver report."""
-        report = Report(count=self.count, pipeline=self.pipeline)
+        states: List[Tuple[Tuple[float, int], State]] = []  # The first item in tuple is used for sorting.
+        heapq_counter = 0  # Making sure the first resolved state takes precedence when adding to the report.
 
         self._init_context()
         with Unit.assigned_context(self.context), self.predictor.assigned_context(self.context):
             for state in self._do_resolve_states(with_devel=with_devel, user_stack_scoring=user_stack_scoring):
-                product = Product.from_final_state(context=self.context, state=state)
-                if report.add_product(product):
-                    # An optimization - we do not need to run wraps if the product is not part of the final result.
-                    self._run_wraps(state)
+                item = ((state.score, heapq_counter), state)
+                heapq_counter -= 1
 
-            if report.product_count() == 0:
+                if len(states) >= self.count:
+                    heapq.heappushpop(states, item)
+                else:
+                    heapq.heappush(states, item)
+
+            if len(states) == 0:
                 msg = (
                     f"Resolver did not find any stack that would satisfy requirements and stack "
                     f"characteristics given the time allocated"
@@ -1308,18 +1334,27 @@ class Resolver:
                         "link": link,
                     }
                 )
-                raise CannotProduceStack(msg + f"- see {link}", stack_info=self.context.stack_info)
+                raise CannotProduceStack(msg + f" - see {link}", stack_info=self.context.stack_info)
 
-            if self.context.stack_info:
-                report.set_stack_info(self.context.stack_info)
+            for item in states:
+                self._run_wraps(item[1])
+
+            report = Report(
+                products=[
+                    Product.from_final_state(context=self.context, state=s[1])
+                    for s in sorted(states, key=lambda s: s[0], reverse=True)
+                ],
+                pipeline=self.pipeline,
+                resolver_iterations=self.context.iteration,
+                accepted_final_states_count=self.context.accepted_final_states_count,
+                discarded_final_states_count=self.context.discarded_final_states_count,
+                stack_info=self.context.stack_info,
+            )
 
             self.predictor.post_run_report(report)
             self.pipeline.call_post_run_report(report)
 
-        report.resolver_iterations = self.context.iteration
-        report.accepted_final_states_count = self.context.accepted_final_states_count
-        report.discarded_final_states_count = self.context.discarded_final_states_count
-        return report
+            return report
 
     def plot(self) -> matplotlib.figure.Figure:
         """Plot history captured during the resolution process."""
@@ -1377,12 +1412,14 @@ class Resolver:
         beam_width: Optional[int] = None,
         count: int = DEFAULT_COUNT,
         graph: Optional[GraphDatabase] = None,
+        labels: Optional[Dict[str, str]] = None,
         library_usage: Optional[Dict[str, Any]] = None,
         limit: int = DEFAULT_LIMIT,
         limit_latest_versions: Optional[int] = DEFAULT_LIMIT_LATEST_VERSIONS,
         project: Project,
         recommendation_type: RecommendationType,
         pipeline_config: Optional[Union[PipelineConfig, Dict[str, Any]]] = None,
+        prescription: Optional["Prescription"] = None,
         cli_parameters: Optional[Dict[str, Any]] = None,
     ) -> "Resolver":
         """Get instance of resolver based on the project given to recommend software stacks."""
@@ -1394,10 +1431,15 @@ class Resolver:
             pipeline = PipelineBuilder.get_adviser_pipeline_config(
                 recommendation_type=recommendation_type,
                 project=project,
+                labels=labels or {},
                 library_usage=library_usage,
                 graph=graph,
+                prescription=prescription,
+                cli_parameters=cli_parameters or {},
             )
         else:
+            assert prescription is None, "Cannot supply prescription and pipeline config at the same time"
+
             if isinstance(pipeline_config, PipelineConfig):
                 pipeline = pipeline_config
             elif isinstance(pipeline_config, dict):
@@ -1409,6 +1451,7 @@ class Resolver:
             beam_width=beam_width,
             count=count,
             graph=graph,
+            labels=labels or {},
             library_usage=library_usage,
             limit_latest_versions=limit_latest_versions,
             limit=limit,
@@ -1416,6 +1459,7 @@ class Resolver:
             predictor=predictor,
             project=project,
             recommendation_type=recommendation_type,
+            prescription=prescription,
             cli_parameters=cli_parameters or {},
         )
 
@@ -1427,11 +1471,13 @@ class Resolver:
         beam_width: Optional[int] = None,
         count: int = DEFAULT_COUNT,
         graph: Optional[GraphDatabase] = None,
+        labels: Optional[Dict[str, str]] = None,
         library_usage: Optional[Dict[str, Any]] = None,
         limit_latest_versions: Optional[int] = DEFAULT_LIMIT_LATEST_VERSIONS,
         project: Project,
         decision_type: DecisionType,
         pipeline_config: Optional[Union[PipelineConfig, Dict[str, Any]]] = None,
+        prescription: Optional["Prescription"] = None,
         cli_parameters: Optional[Dict[str, Any]] = None,
     ) -> "Resolver":
         """Get instance of resolver based on the project given to run dependency monkey."""
@@ -1444,9 +1490,14 @@ class Resolver:
                 decision_type=decision_type,
                 graph=graph,
                 project=project,
+                labels=labels or {},
                 library_usage=library_usage,
+                prescription=prescription,
+                cli_parameters=cli_parameters or {},
             )
         else:
+            assert prescription is None, "Cannot supply prescription and pipeline config at the same time"
+
             if isinstance(pipeline_config, PipelineConfig):
                 pipeline = pipeline_config
             elif isinstance(pipeline_config, dict):
@@ -1459,11 +1510,13 @@ class Resolver:
             count=count,
             limit=count,  # always match as limit is always same as count for Dependency Monkey.
             graph=graph,
+            labels=labels or {},
             library_usage=library_usage,
             limit_latest_versions=limit_latest_versions,
             pipeline=pipeline,
             predictor=predictor,
             project=project,
             decision_type=decision_type,
+            prescription=prescription,
             cli_parameters=cli_parameters or {},
         )
