@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # thoth-adviser
-# Copyright(C) 2019, 2020 Fridolin Pokorny
+# Copyright(C) 2019-2021 Fridolin Pokorny
 #
 # This program is free software: you can redistribute it and / or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,44 +17,43 @@
 
 """Implementation of sieves used in adviser pipeline."""
 
-from .abi_compat import AbiCompatibilitySieve
-from .backports import Enum34BackportSieve
-from .backports import Functools32BackportSieve
-from .backports import ImportlibMetadataBackportSieve
-from .backports import ImportlibResourcesBackportSieve
-from .backports import MockBackportSieve
+from .constraints import ConstraintsSieve
+from .experimental_filter_conf_index import FilterConfiguredIndexSieve
+from .experimental_package_index import PackageIndexConfigurationSieve
+from .experimental_prereleases import SelectiveCutPreReleasesSieve
 from .filter_index import FilterIndexSieve
 from .index_enabled import PackageIndexSieve
+from .legacy_version import LegacyVersionSieve
 from .locked import CutLockedSieve
-from .pandas import PandasPy36Sieve
 from .prereleases import CutPreReleasesSieve
-from .setuptools import Py36SetuptoolsSieve
+from .rules import SolverRulesSieve
 from .solved import SolvedSieve
+from .tensorflow import TensorFlow240AVX2IllegalInstructionSieve
 from .tensorflow import TensorFlowAPISieve
-from .tensorflow import TensorFlowCUDASieve
-from .tensorflow import TensorFlowPython39Sieve
+from .thoth_s2i_abi_compat import ThothS2IAbiCompatibilitySieve
+from .thoth_s2i_packages import ThothS2IPackagesSieve
+from .update import PackageUpdateSieve
 from .version_constraint import VersionConstraintSieve
 
-
 # Relative ordering of units is relevant, as the order specifies order
 # in which the asked to be registered - any dependencies between them
 # can be mentioned here.
 __all__ = [
+    "PackageUpdateSieve",
+    "LegacyVersionSieve",
     "CutPreReleasesSieve",
+    "ConstraintsSieve",
+    "SelectiveCutPreReleasesSieve",
+    "FilterConfiguredIndexSieve",
+    "PackageIndexConfigurationSieve",
     "CutLockedSieve",
     "PackageIndexSieve",
     "SolvedSieve",
+    "SolverRulesSieve",
     "VersionConstraintSieve",
-    "AbiCompatibilitySieve",
+    "ThothS2IPackagesSieve",
+    "ThothS2IAbiCompatibilitySieve",
     "FilterIndexSieve",
-    "Enum34BackportSieve",
-    "Functools32BackportSieve",
-    "ImportlibMetadataBackportSieve",
-    "ImportlibResourcesBackportSieve",
-    "MockBackportSieve",
-    "Py36SetuptoolsSieve",
+    "TensorFlow240AVX2IllegalInstructionSieve",
     "TensorFlowAPISieve",
-    "TensorFlowCUDASieve",
-    "TensorFlowPython39Sieve",
-    "PandasPy36Sieve",
 ]
