@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # thoth-adviser
-# Copyright(C) 2019, 2020 Fridolin Pokorny
+# Copyright(C) 2019 - 2021 Fridolin Pokorny
 #
 # This program is free software: you can redistribute it and / or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@ import logging
 from typing import Any
 from typing import Dict
 from typing import Generator
-from typing import Optional
 from typing import Set
 from typing import Tuple
 from typing import TYPE_CHECKING
@@ -50,16 +49,18 @@ class SolvedSieve(Sieve):
     CONFIGURATION_SCHEMA: Schema = Schema(
         {Required("package_name"): SchemaAny(str, None), Required("without_error"): bool}
     )
-    _JUSTIFICATION_LINK = jl("buildtime_error")
+    _JUSTIFICATION_LINK = jl("install_error")
 
     _messages_logged = attr.ib(type=Set[Tuple[str, str, str]], factory=set, init=False)
 
     @classmethod
-    def should_include(cls, builder_context: "PipelineBuilderContext") -> Optional[Dict[str, Any]]:
+    def should_include(cls, builder_context: "PipelineBuilderContext") -> Generator[Dict[str, Any], None, None]:
         """Include solved pipeline sieve for adviser or Dependency Monkey on pipeline creation."""
         if not builder_context.is_included(cls):
-            return {}
+            yield {}
+            return None
 
+        yield from ()
         return None
 
     def pre_run(self) -> None:
@@ -85,15 +86,17 @@ class SolvedSieve(Sieve):
                 _LOGGER.debug(
                     "Removing package %r as it was not solved: %s",
                     package_tuple,
-                    str(exc),
+                    exc,
                 )
                 continue
 
             if has_error and self.configuration["without_error"]:
                 if package_tuple not in self._messages_logged:
                     self._messages_logged.add(package_tuple)
-                    message = f"Removing package {package_tuple} due to build time error in the software environment"
-                    _LOGGER.warning("%s - see %s", message, self._JUSTIFICATION_LINK)
+                    message = (
+                        f"Removing package {package_tuple} due to installation time error in the software environment"
+                    )
+                    _LOGGER.debug("%s - see %s", message, self._JUSTIFICATION_LINK)
                     self.context.stack_info.append(
                         {
                             "type": "WARNING",
