@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # thoth-adviser
-# Copyright(C) 2020 Fridolin Pokorny
+# Copyright(C) 2020 - 2021 Fridolin Pokorny
 #
 # This program is free software: you can redistribute it and / or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,13 +17,11 @@
 
 """Implementation of sieves used, specific for TensorFlow."""
 
+from .tf_240_avx2 import TensorFlow240AVX2IllegalInstructionSieve
 from .tf_api import TensorFlowAPISieve
-from .tf_cuda import TensorFlowCUDASieve
-from .tf_py39 import TensorFlowPython39Sieve
 
 
 __all__ = [
+    "TensorFlow240AVX2IllegalInstructionSieve",
     "TensorFlowAPISieve",
-    "TensorFlowCUDASieve",
-    "TensorFlowPython39Sieve",
 ]
