@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # thoth-adviser
-# Copyright(C) 2019, 2020 Fridolin Pokorny
+# Copyright(C) 2019 - 2021 Fridolin Pokorny
 #
 # This program is free software: you can redistribute it and / or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ import logging
 
 from typing import Any
 from typing import Dict
+from typing import Generator
 from typing import List
 from typing import Optional
 from typing import Set
@@ -62,17 +63,17 @@ class CvePenalizationStep(Step):
     _messages_logged = attr.ib(type=Set[Tuple[str, str, str]], factory=set, init=False)
 
     @classmethod
-    def should_include(cls, builder_context: "PipelineBuilderContext") -> Optional[Dict[str, Any]]:
+    def should_include(cls, builder_context: "PipelineBuilderContext") -> Generator[Dict[str, Any], None, None]:
         """Remove CVEs only for advised stacks."""
-        if not builder_context.is_adviser_pipeline():
+        if (
+            builder_context.is_adviser_pipeline()
+            and builder_context.recommendation_type != RecommendationType.LATEST
+            and not builder_context.is_included(cls)
+        ):
+            yield {}
             return None
 
-        if builder_context.recommendation_type == RecommendationType.LATEST:
-            return None
-
-        if not builder_context.is_included(cls):
-            return {}
-
+        yield from ()
         return None
 
     def pre_run(self) -> None:
@@ -88,7 +89,7 @@ class CvePenalizationStep(Step):
                 package_version=package_version.locked_version,
             )
         except NotFoundError as exc:
-            _LOGGER.warning("Package %r in version %r not found: %r", str(exc))
+            _LOGGER.warning("Package %r in version %r not found: %r", exc)
             return None
 
         if cve_records:
@@ -99,22 +100,37 @@ class CvePenalizationStep(Step):
                 if package_version_tuple not in self._messages_logged:
                     self._messages_logged.add(package_version_tuple)
                     for cve_record in cve_records:
+                        message = (
+                            f"Skipping including package {package_version_tuple!r} as a CVE "
+                            f"{cve_record['cve_id']!r} was found"
+                        )
                         _LOGGER.warning(
-                            "Skipping including package %r as a CVE %s was found: %s",
-                            package_version_tuple,
-                            cve_record.get("cve_name") or cve_record.get("cve_id"),
-                            cve_record["advisory"],
+                            "%s: %s",
+                            message,
+                            cve_record["details"],
+                        )
+
+                        self.context.stack_info.append(
+                            {"type": "WARNING", "message": message, "link": self._JUSTIFICATION_LINK}
                         )
 
                 raise NotAcceptable
 
             penalization = len(cve_records) * self.configuration["cve_penalization"]
 
-            # Note down package causing this CVE.
-            for record in cve_records:
-                record["package_name"] = package_version.name
-                record["link"] = self._JUSTIFICATION_LINK
+            justification = []
+            for cve_record in cve_records:
+                message = f"Package  {package_version_tuple!r} has a CVE {cve_record['cve_id']!r}"
+                justification.append(
+                    {
+                        "package_name": package_version.name,
+                        "link": self._JUSTIFICATION_LINK,
+                        "advisory": cve_record["details"],
+                        "message": message,
+                        "type": "WARNING",
+                    }
+                )
 
-            return penalization, cve_records
+            return max(penalization, -1.0), justification
 
         return None
